package inf101.v18.rogue101.enemies;

import inf101.v18.grid.Location;
import inf101.v18.rogue101.game.Game;
import inf101.v18.rogue101.items.container.Backpack;
import inf101.v18.rogue101.object.Item;
import inf101.v18.rogue101.object.NonPlayerCharacter;
import inf101.v18.rogue101.state.Sound;

public abstract class Enemy implements NonPlayerCharacter {

    protected int hp = getMaxHealth();
    protected final Backpack backpack = new Backpack();
    protected Location currentLocation;

    @Override
    public Item getItem(Class<?> type) {
        for (Item item : backpack.getContent()) {
            if (type.isInstance(item)) {
                return item;
            }
        }
        return null;
    }

    @Override
    public int handleDamage(Game game, Item source, int amount) {
        hp -= amount;
        if (hp < 0) {
            Sound.NPC_DEATH.play();
        }
        if (hp < 0 && backpack.size() > 0) { //Will be useful in a dungeon with several bosses
            boolean dropped = false;
            for (Item item : backpack.getContent()) {
                if (game.dropAt(currentLocation, item)) {
                    dropped = true;
                }
            }
            if (dropped) {
                game.displayMessage(getName() + " dropped something");
            }
        }
        return amount;
    }

}
