import inf101.v18.grid.GridDirection;
import inf101.v18.grid.IArea;
import inf101.v18.grid.Location;
import inf101.v18.util.Generator;
import inf101.v18.util.generators.AreaGenerator;
import inf101.v18.util.generators.LocationGenerator;
import org.junit.Test;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

public class AreaRetting {
    private static final int N = 10000;

    private Generator<IArea> areaGen = new AreaGenerator();

    public void allAreaProperties() {

    }

    public void canGoProperty(Location l, GridDirection dir) {
        int x = l.getX() + dir.getDx();
        int y = l.getY() + dir.getDy();
        assertEquals(l.getArea().contains(x, y), l.canGo(dir));
        try {
            assertNotNull(l.go(dir));
            assertTrue(String.format("Expected true: %s.canGo(%s) for %d,%d", l, dir, x, y), l.canGo(dir));
        } catch (IndexOutOfBoundsException e) {
            assertFalse(String.format("Expected false: %s.canGo(%s) for %d,%d", l, dir, x, y), l.canGo(dir));
        }
    }

    public void distanceProperty(Location l1, Location l2) {
        assertEquals(l1.gridDistanceTo(l2), l2.gridDistanceTo(l1));
        assertEquals(l1.stepDistanceTo(l2), l2.stepDistanceTo(l1));
        assertTrue(l1.gridDistanceTo(l2) <= l1.stepDistanceTo(l2));
        assertTrue(l1.gridDistanceTo(l2) <= l1.geometricDistanceTo(l2));
    }

    public void gridLineProperty(Location l1, Location l2) {
        // System.out.println(l1.toString() + " .. " + l2.toString());
        List<Location> line = l1.gridLineTo(l2);
        assertEquals(l1.gridDistanceTo(l2), line.size());
        Location last = l1;
        for (Location l : line) {
            assertEquals(1, last.gridDistanceTo(l));
            last = l;
        }
        assertEquals(l2, last);
    }

    @Test
    public void gridLineTest() {
        for (int i = 0; i < 10; i++) {
            IArea area = areaGen.generate();
            Generator<Location> lGen = new LocationGenerator(area);
            for (int j = 0; j < N; j++) {
                Location l1 = lGen.generate();
                Location l2 = lGen.generate();
                distanceProperty(l1, l2);
                gridLineProperty(l1, l2);
            }
        }
    }

    @Test
    public void locationsTest() {
        for (int i = 0; i < 10; i++) {
            IArea area = areaGen.generate();
            for (Location l : area) {
                neighboursDistProperty(l);
                neighboursSymmetryProperty(l);
                for (GridDirection d : GridDirection.EIGHT_DIRECTIONS)
                    canGoProperty(l, d);
            }
        }
    }

    public void neighboursDistProperty(Location loc) {
        for (Location l : loc.allNeighbours()) {
            assertEquals(1, loc.gridDistanceTo(l));
        }
    }

    public void neighboursSymmetryProperty(Location loc) {
        for (Location l : loc.allNeighbours()) {
            assertTrue("My neighbour should have me as a neighbour", l.allNeighbours().contains(loc));
        }
    }

    @Test
    public void uniqueLocations() {
        for (int i = 0; i < N / 10; i++) {
            IArea area = areaGen.generate();
            uniqueLocationsProperty(area);
        }
    }

    public void uniqueLocationsProperty(IArea area) {
        Set<Location> set = new HashSet<>();
        for (Location l : area) {
            assertTrue("Location should be unique: " + l, set.add(l));
        }
    }

    @Test
    public void validLocations() {
        for (int i = 0; i < N / 10; i++) {
            IArea area = areaGen.generate();
            validLocationsProperty(area);
        }
    }

    public void validLocationsProperty(IArea area) {
        for (Location l : area) {
            assertTrue("Location should be in area: " + l, area.contains(l));
            assertTrue("Location should be in area: " + l, area.contains(l.getX(), l.getY()));
        }
    }
}
